@@ -23,11 +23,11 @@ import java.io.DataOutputStream;
 import java.io.IOException;
 import java.io.OutputStream;
 import java.io.Serializable;
-import java.nio.ByteBuffer;
 import java.util.ArrayList;
 
 import org.apache.uima.cas.CASRuntimeException;
 import org.apache.uima.cas.Marker;
+import org.apache.uima.util.CasIOUtils;
 
 /**
  * This object has 2 purposes.
@@ -150,29 +150,7 @@ public class CASSerializer implements Serializable {
     this.longHeapArray = new long[longHeapSize];
     System.arraycopy(cas.getLongHeap().heap, 0, this.longHeapArray, 0, longHeapSize);
   }
-  
-  // version
-  // encode: bits 7 6 5 4 3 2 1 0
-  //                        0 0 1 = no delta, no compression
-  //                        0 1 - = delta, no compression
-  //                        1 d - = compression, w/wo delta
-
-  static void outputVersion(int version, DataOutputStream dos) throws IOException {
-    // output the key and version number
-
-    byte[] uima = new byte[4];
-    uima[0] = 85; // U
-    uima[1] = 73; // I
-    uima[2] = 77; // M
-    uima[3] = 65; // A
-
-    ByteBuffer buf = ByteBuffer.wrap(uima);
-    int key = buf.asIntBuffer().get();
-
-    dos.writeInt(key);
-    dos.writeInt(version);
-  }
-  
+    
   private void outputStringHeap(DataOutputStream dos, CASImpl cas, StringHeapDeserializationHelper shdh) throws IOException {
     // output the strings
 
@@ -227,6 +205,10 @@ public class CASSerializer implements Serializable {
       dos.writeInt(shdh.refHeap[i + StringHeapDeserializationHelper.CHAR_HEAP_STRLEN_OFFSET]);
     }
   }
+  
+  void addTsiCAS(CASImpl cas, OutputStream ostream) {
+    
+  }
 
   /**
    * Serializes the CAS data and writes it to the output stream.
@@ -257,17 +239,27 @@ public class CASSerializer implements Serializable {
    * @param ostream -
    */
   public void addCAS(CASImpl cas, OutputStream ostream) {
- 
-    try {
+    addCAS(cas, ostream, false);
+  }
+    
+  public void addCAS(CASImpl cas, OutputStream ostream, boolean includeTsi) { 
+  try {
 
       DataOutputStream dos = new DataOutputStream(ostream);
 
+      // output the key and version number
+      CommonSerDes.createHeader()
+      .seqVer(1)     // not a delta, set version 1 for UIMA-4743 to inform receiver that we can handle version 1 format
+      .typeSystemIndexDefIncluded(includeTsi)
+      .write(dos);
+      
+      if (includeTsi) {
+        CasIOUtils.writeTypeSystem(cas, ostream, true);
+      }
+
       // get the indexed FSs
       this.fsIndex = cas.getIndexedFSs();
-
-      // output the key and version number
-      outputVersion(1, dos);
-      
+     
       // output the FS heap
       final int heapSize = cas.getHeap().getCellsUsed();
       dos.writeInt(heapSize);
@@ -430,10 +422,10 @@ public class CASSerializer implements Serializable {
       // get the indexed FSs
       this.fsIndex = cas.getDeltaIndexedFSs(mark);
       
-      // output the key and version number
-      //1 = current full serialization; 2 = delta format 
-      //perhaps this should be split into 2 bytes for version and 2 bytes for format.
-      outputVersion(2, dos);
+      CommonSerDes.createHeader()
+      .delta()                                // is delta
+      .seqVer(1)
+       .write(dos);    
 
       // output the new FS heap cells
       final int heapSize = cas.getHeap().getCellsUsed() - mark.nextFSId;
@@ -557,7 +549,7 @@ public class CASSerializer implements Serializable {
 	    }
 
       // word alignment
-      align = (4 - (byteheapsz % 4)) % 4;
+      align = (4 - (byteHeapModifiedAddrs.length % 4)) % 4;
       for (int i = 0; i < align; i++) {
         dos.writeByte(0);
       }
@@ -567,25 +559,27 @@ public class CASSerializer implements Serializable {
       short[] shortValues = new short[shortHeapModifiedAddrs.length];
       dos.writeInt(shortHeapModifiedAddrs.length);
       for (int i=0; i < shortHeapModifiedAddrs.length; i++) {
-    	dos.writeShort(shortHeapModifiedAddrs[i]);
-    	shortValues[i] = cas.getShortHeap().getHeapValue(shortHeapModifiedAddrs[i]);
+        int modifiedAddr = shortHeapModifiedAddrs[i]; 
+        dos.writeInt(modifiedAddr);
+      	shortValues[i] = cas.getShortHeap().getHeapValue(modifiedAddr);
       }
       for (int i=0; i < shortValues.length; i++) {  
     	  dos.writeShort(cas.getShortHeap().getHeapValue(shortHeapModifiedAddrs[i]));
 	  }
       
       // word alignment
-      if (shortheapsz % 2 != 0) {
+      if (shortHeapModifiedAddrs.length % 2 != 0) {
         dos.writeShort(0);
       }
       
       // 64 bit heap modified cells
-      int[] longHeapModifiedAddrs = cas.getModifiedShortHeapAddrs().toArray();
+      int[] longHeapModifiedAddrs = cas.getModifiedLongHeapAddrs().toArray(); // https://issues.apache.org/jira/browse/UIMA-4743
       long[] longValues = new long[longHeapModifiedAddrs.length];
       dos.writeInt(longHeapModifiedAddrs.length);
       for (int i=0; i < longHeapModifiedAddrs.length; i++) {
-    	dos.writeShort(longHeapModifiedAddrs[i]);
-    	longValues[i] = cas.getLongHeap().getHeapValue(longHeapModifiedAddrs[i]);
+        int modifiedAddr = longHeapModifiedAddrs[i];
+        dos.writeInt(modifiedAddr);
+        longValues[i] = cas.getLongHeap().getHeapValue(modifiedAddr);
       }
       for (int i=0; i < longValues.length; i++) {  
     	  dos.writeLong(cas.getLongHeap().getHeapValue(longHeapModifiedAddrs[i]));
