@@ -693,8 +693,8 @@ public class CasSerializerSupport {
         String typeName = tsi.ll_getTypeForCode(typeCode).getName();
         XmlElementName newXel = csss.uimaTypeName2XmiElementName(typeName);
 
-        if (!needNameSpaces) {
-          csss.checkForNameCollision(newXel);
+        if (!needNameSpaces) {   // means if name spaces are not not always needed, then we have to check for collision
+          csss.checkForNameCollision(newXel);   // executed for JSON code
         }        
         typeCode2namespaceNames[typeCode] = newXel;
       }  
@@ -850,7 +850,9 @@ public class CasSerializerSupport {
               continue;  // don't enqueue this type because it's filtered out
             }
           }
-          enqueue(fsRef);  
+          if (fsRef != CASImpl.NULL) {      // null feature values do not refer to any other FS
+            enqueue(fsRef);
+          }
         }
         return;
       }
