@@ -84,6 +84,8 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
   private final boolean isBounded;
   private boolean isListForm = false;
   private final boolean isBeginEndCompare;
+  
+  private final int startId;
 
   
   /**
@@ -103,7 +105,7 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
    * @param ambiguous true means normal iteration, 
    *                  false means to skip annotations whose begin lies between previous begin (inclusive) and end (exclusive)
    * @param strict true means to skip annotations whose end is greater than the bounding end position (ignoring type priorities)
-   * @param isBounded false means its an unambiguous iterator with no bounds narrowing; ambiguous taken to be false
+   * @param isBounded false means it's an unambiguous iterator with no bounds narrowing; ambiguous taken to be false
    * @param fsIndexRepo the index repository for this iterator
    */
   Subiterator(
@@ -129,6 +131,7 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
     this.fsIndexRepo = fsIndexRepo;
     
     moveToStart();
+    startId = (isValid()) ? ((FeatureStructureImpl)get()).getAddress() : 0;
   }
     
   
@@ -140,7 +143,7 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
    * 
    */
   private void convertToListForm() {
-    moveToStart();  // moves to the start annotation, including moving past equals for annot style, and accomodating strict
+    moveToStart();  // moves to the start annotation, including moving past equals for annot style, and accommodating strict
     this.list = new ArrayList<T>();
     while (isValid()) {
       prevEnd = it.getEnd();
@@ -258,6 +261,7 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
    * 
    * @see org.apache.uima.cas.FSIterator#isValid()
    */
+  @Override
   public boolean isValid() {
     return isListForm ? 
         (this.pos >= 0) && (this.pos < this.list.size()) :
@@ -273,6 +277,7 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
    * 
    * @see org.apache.uima.cas.FSIterator#get()
    */
+  @Override
   public T get() throws NoSuchElementException {
     if (isListForm) {
       if ((this.pos >= 0) && (this.pos < this.list.size())) {
@@ -291,6 +296,7 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
    * 
    * @see org.apache.uima.cas.FSIterator#moveToNext()
    */
+  @Override
   public void moveToNext() {
     if (isListForm) {
       ++this.pos;
@@ -305,7 +311,14 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
     }
 
     adjustForStrictForward();
-    setPrevEnd();
+    
+    // stop in bounded case if out of bounds going forwards UIMA-5063
+    if (isBounded && it.isValid() && (it.get().getBegin() > boundingEnd)) {
+      it.moveToLast();
+      it.moveToNext();  // mark invalid
+    } else {
+      setPrevEnd();
+    }
   }
 
   /*
@@ -313,6 +326,7 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
    * 
    * @see org.apache.uima.cas.FSIterator#moveToPrevious()
    */
+  @Override
   public void moveToPrevious() {
     if (isListForm) {
       --this.pos;
@@ -328,8 +342,14 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
       return;
     }
     
-    it.moveToPrevious();
-    adjustForStrictBackward();
+    // stop in bounded case if out of bounds going backwards UIMA-5063
+    if (isBounded && isValid() && ((FeatureStructureImpl)it.get()).getAddress() == startId) {
+      it.moveToFirst();
+      it.moveToPrevious();  // make it invalid
+    } else {
+      it.moveToPrevious();
+      adjustForStrictBackward();
+    }
   }
 
   /*
@@ -337,6 +357,7 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
    * 
    * @see org.apache.uima.cas.FSIterator#moveToFirst()
    */
+  @Override
   public void moveToFirst() {
     if (isListForm) {
       this.pos = 0;
@@ -350,6 +371,7 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
    * 
    * @see org.apache.uima.cas.FSIterator#moveToLast()
    */
+  @Override
   public void moveToLast() {
     if (isListForm) {
       this.pos = this.list.size() - 1;
@@ -392,6 +414,7 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
    * 
    * @see org.apache.uima.cas.FSIterator#moveTo(org.apache.uima.cas.FeatureStructure)
    */
+  @Override
   public void moveTo(FeatureStructure fs) {
     AnnotationFS fsa = (AnnotationFS) fs;
     if (!ambiguous && !isListForm) {  // unambiguous must be in list form
@@ -436,10 +459,7 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
       // is ambiguous, may be strict, always bounded (either by annotation or begin / end
       it.moveTo(fs);
       if (isBeginEndCompare) {
-        it.moveTo(fs);
         adjustAfterMoveToForBeginEndComparator(fsa);
-      } else {  // is begin/end bounding
-        it.moveTo(fs);
       }
       adjustForStrictForward();
     }
@@ -449,7 +469,7 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
    * @see org.apache.uima.cas.impl.FSIteratorImplBase#moveTo(java.util.Comparator)
    */
   @Override
-  <TT extends AnnotationFS> void moveTo(final int begin, final int end) {
+  void moveTo(final int begin, final int end) {
     if (!ambiguous && !isListForm) {  // unambiguous must be in list form
       convertToListForm();
     }
@@ -496,6 +516,7 @@ public class Subiterator<T extends AnnotationFS> extends FSIteratorImplBase<T> {
    * 
    * @see org.apache.uima.cas.FSIterator#copy()
    */
+  @Override
   public FSIterator<T> copy() {
     Subiterator<T> copy = new Subiterator<T>(
         this.it, this.boundingAnnotation, this.boundingBegin, this.boundingEnd, this.ambiguous, this.strict, this.isBounded, this.fsIndexRepo);
