@@ -28,6 +28,7 @@ import java.util.Collections;
 import java.util.HashMap;
 import java.util.List;
 import java.util.Map;
+import java.util.Set;
 import java.util.concurrent.ConcurrentHashMap;
 
 import org.apache.uima.UIMAFramework;
@@ -143,6 +144,12 @@ public class ResourceManager_impl implements ResourceManager {
   //   of making this a ConcurrentHashMap  (March 2014)
   final private Map<String,XMLizable> importCache = Collections.synchronizedMap(new HashMap<String,XMLizable>());
   
+  /**
+   * Cache of imported descriptor URLs from which the parsed objects in importCache
+   * were created, so that these URLs are not re-parsed if the same URL is imported again.
+   */
+  final private Map<String,Set<String>> importUrlsCache = Collections.synchronizedMap(new HashMap<String,Set<String>>());
+  
   /**
    * Creates a new <code>ResourceManager_impl</code>.
    */
@@ -713,4 +720,7 @@ public class ResourceManager_impl implements ResourceManager {
     return importCache;
   }
 
+  public Map<String, Set<String>> getImportUrlsCache() {
+    return importUrlsCache;
+  }
 }
